function [out,t,Aoy,Aox,ry,rx] = MDDT(in,op,z,lambda,delta0y,delta0x,deltaZy,deltaZx,q,av,reg,Ay,Ax)
% MDDT() realizes the pixel-to-pixel discrete diffraction propagation in the spatial domain described in [1]. 
% This function calculates the sensor plane wave field using the input (complex-valued) object or
% the object reconstruction from the given/computed complex-valued observation by the Tikhonov regularized inverse
% or via the truncated SVD.
%
% Notation:
% [out,t,Aoy,Aox] = MDDT(in,op,z,lambda,delta0y,delta0x,deltaZy,deltaZx,q,av,reg,Ay,Ax)
% ------------------------------- input -----------------------------------
% in                    the input (object or sensor plane) wave field distribution depending on
%                       the requied operation op of the forward/backward diffraction propagation
% op                    this parameter determines the perfomed operation:
%                       If op = 0, MDDT() realizes the forward wave field propagation and results in the sensor 
%                       plane wave field. In this case the input parameter must be the object wave field.
%                       If op = 1, MDDT() realizes the Tikhonov regularize inverse and results in the estimate
%                       of the object wave field. Here the input parameter must be the sensor plane wave field. 
%                       If op = 2, MDDT() realizes the regularization by truncation. 
%                       The input parameter must be the sensor plane wave field.
% z                     propagation distance between the object and sensor planes [m]
% lambda                wavelength [m]
% delta0y               pixel size of the discrete object wave field w.r.t. rows [m]
% delta0x               pixel size of the discrete object wave field w.r.t. columns [m]
% deltaZy               pitch size of the discrete sensor plane wave field w.r.t. rows [m] 
% deltaZx               pitch size of the discrete sensor plane wave field w.r.t. columns [m]
% q = [qy,qx]           redundancy of the sensor (the ratio of the sesor plane wave field distribution 
%                       w.r.t. the object one) in the y- and x-direction, respectively (by default for the same 
%                       size of the object and sensor plane wave fields qy = 1, qx = 1, must be qy >= 1, qx >= 1).
% av                    this parameters determines which matrices to use in the M-DDT model (optional): 
%                       If av = 1 (by default) - the averaged transform matrices.
%                       If av = 0 - the non-averaged transform matrices.
% reg                   If op = 1 and the Tikhonov regularized inverse is about to be performed, then
%                       reg must be the regularization parameter (by default reg=1e-24)
%                       If op = 2 and the trunctated SVD is chosen, then
%                       reg = [regY,regX] - is the number of used singular values in the transform matrices
%                       w.r.t. the rows and columns of the object 2D distribution, respectively.
%                       (optional, by default it is the numerical rank of the M-DDT transform matrices)
% Ay, Ax                the M-DDT transform matrices used for accelaration of calculation (optional)
% ry, rx                rank of the M-DDT transform matrices 
% ------------------------------- output ----------------------------------
% out                   the computed wave field: if op = 0 - the sensor plane wave field 
%                                                if op = 1 or op = 2 - the object estimate 
% t                     the computational time in seconds (optional)
% Aoy,Aox               the M-DDT transform matrices for the chosen parameters
%
% Example #1:
% Calculation of the sensor plane wave field uz of the size M1 x M2 using the amplitude object u0 of the size N1 x N2
% with different rectangular pixels at the object and sensor planes of the size delta0y x delta0x and deltaZy x deltaZx, respectively. 
% u0 = im2double(imread('Lena256.png'));                 % read the test-image
% [N1,N2,N3] = size(u0);                                 % image size
% lambda = 6328e-10;                                     % wavelength (red laser)
% delta0y = 6e-6; delta0x = 8e-6;                        % rectangular pixel size for the object
% deltaZy = delta0y; deltaZx = delta0x;                  % and for the sensor plane wave fields
% M1 =  N1; M2 = N2;                                     % the sensor plane wave field is taken of the same size as the object size
% z = delta0y*deltaZy*M1/lambda;                         % propagation distance
% uz = MDDT(u0,0,z,lambda,delta0y,delta0x,deltaZy,deltaZx); % the complex-valued sensor plane wave field
%
% Example #2:
% Simulation of the M-DDT pixel-to-pixel diffraction propagation of rectangular pixel and images of different sizes at the object and sensor planes.  
% The discrete amplitude-only object u00 is of the size 512x256 with the pixels of the size 5x8 (microns).
% The sesnor plane discrete wave field uz (the result of the forward propagation) is calculated of the size 616x334 with the pixel size 10x6 (microns).
% The reconstruction of the object (backward diffraction propagation) is performed by the Tikhonov regularized inverse and truncated SVD.
% u0 = im2double(imread('Baboon512.png'));
% u00 = u0(:,129:384);                                   % rectangular amplitude-only object of the size 512x256
% [N1,N2,N3] = size(u00);
% lambda = 532e-9;                                       % wavelength (green laser)
% delta0x = 8e-6; delta0y = 5e-6;                        % rectangular pixel size for the object
% deltaZy = 10e-6; deltaZx = 6e-6;                       % and for the sensor plane wave fields
% q = [1.2,1.3];                                         % redundancy of the senor to the size 616x334
% M1 =  q(1)*N1; M2 = q(2)*N2;                           % the sensor plane wave field is taken of the same size as the object size
% z = delta0y*deltaZy*M1/lambda;                         % propagation distance
% [uz,~,Ay,Ax] = MDDT(u00,0,z,lambda,delta0y,delta0x,deltaZy,deltaZx,q); % the estimate of the sensor plane wave field
%
% u01 = MDDT(uz,1,z,lambda,delta0y,delta0x,deltaZy,deltaZx,q,[],1e-24,Ay,Ax); % Calculation of the object reconstruction by the Tikhonov regularized inverse, 
%                                                        % the regularization parameter is equal to 10^(-24). Since the averaging in the M-DDT transform 
%                                                        % matrices is used by default we leave the corresponding parameter empty. Note! in order to obtain  
%                                                        % the object reconstruction of the original size 512x256 the sensor redundancy q must be determined!
%
% u02 = MDDT(uz,2,z,lambda,delta0y,delta0x,deltaZy,deltaZx,q,[],[512,254],Ay,Ax); % Calculation of the object reconstruction via the truncated SVD (op=2). 
%                                                        % Here we leave 512 and 254 singular vaules of the transform matrices w.r.t. the y- and x-direction.
%                                                        % In doubt keep reg=[] empty to use the numerical rank of the transform matrices as the number of singular values.
%
% Reference:            [1] V. Katkovnik, A. Migukin, and J. Astola, Appl. Opt. 48, 3407-3423 (2009). 
% Authors:              Artem Migukin
% Software version:     2.0, January 2013.
% Required functions:   TransformMatrixMDDT()

% --------------- please do not change the code under this line -----------------------------------------
if (nargin<9 || isempty(q)),    q=[1,1]; end
if length(q)~=2 || ~isreal(q(1)) || ~isnumeric(q(1)), 
    disp('Error: Wrong setting of the parameter q=[qy,qx], Use the default q=[1,1]!'); q = [1,1]; 
end
if q(1) < 1 || q(2) < 1, disp('Error: qy or qx must be not smaller than 1'); q = [1,1]; end
if (nargin<10 || isempty(av)),  av=true; end          % by default the averaging for transform matrix is switched on 

if op == 0                                                                  % forward propagation
    [Ny0_,Nx0_,tmp] = size(in);                                             % 'in' is the object
    Ny0 = ceil(Ny0_/2)*2;Nx0 = ceil(Nx0_/2)*2;
    tmp = zeros(Ny0,Nx0); tmp(Ny0/2-Ny0_/2+1:Ny0/2+Ny0_/2,Nx0/2-Nx0_/2+1:Nx0/2+Nx0_/2)=in; in = tmp;
    Nyz = ceil(Ny0*q(1)/2)*2; Nxz = ceil(Nx0*q(2)/2)*2;                     % make the size of the sensor plane wave field even
else
    [Nyz_,Nxz_,tmp]=size(in);                                               % 'in' is the senser plane wave field   
    Nyz = ceil(Nyz_/2)*2;Nxz = ceil(Nxz_/2)*2;
    tmp = zeros(Nyz,Nxz); tmp(Nyz/2-Nyz_/2+1:Nyz/2+Nyz_/2,Nxz/2-Nxz_/2+1:Nxz/2+Nxz_/2)=in; in = tmp;
    Ny0 = floor(Nyz/q(1)/2)*2; Nx0 = floor(Nxz/q(2)/2)*2; % find even size of the object for the given q=[qy,qx]
    if op==1 
        if (nargin<11 || isempty(reg)),  reg=1e-024; end
        if length(reg)~=1 || reg(1)<=0 || ~isreal(reg(1)) || ~isnumeric(reg(1))
            disp('Error: Wrong setting of the regularization parameter. reg should be a positive real-valued scalar'); 
            out=[];t=0;Aoy=[];Aox=[]; return
        end
    end    
    if op==2 
      if ~(nargin<11 || isempty(reg))
        if length(reg)~=2 || (ceil(reg(1))-floor(reg(1))==1 || ceil(reg(2))-floor(reg(2))==1) || (reg(1)<=0 || reg(2)<=0)
            disp('Error: Wrong setting of the regularization parameter. reg=[regy,regx] should be a vector of two positive integers'); 
            out=[];t=0;Aoy=[];Aox=[]; return
        end
      end
    end
end

if nargin<12 || isempty(Ay)                                                 % calculation of the M-DDT transform matrices
    [Aoy,tmp,ry] = TransformMatrixMDDT(z,lambda,delta0y,deltaZy,Ny0,Nyz,av); t = tmp;
else  tic;  Aoy = Ay;        ry = rank(Ay'*Ay,10e-12);    t=toc; clear Ay
end
if nargin<13 || isempty(Ax)                                                 % calculation of the M-DDT transform matrices
    [Aox,tmp,rx] = TransformMatrixMDDT(z,lambda,delta0x,deltaZx,Nx0,Nxz,av); t = t+tmp;
else  tic;  
    Aox = Ax;        rx = rank(Ax'*Ax,10e-12);    t=t+toc; clear Ax
end

if op == 0
    if nargout > 1,    tic;end                                                  % start a stopwatch timer 
% calculation in the two-matrix form
    out = Aoy*in*(Aox.');
    t = t+toc;% stop the stopwatch timer - obtain the resulting time of calculation
elseif op == 1
    
    if nargout > 1,    tic;end                                                  % start a stopwatch timer 
    % SVD
    [uy,ey,vy]=svd(Aoy);eey = diag(ey); 
    [ux,ex,vx]=svd(Aox);eex = diag(ex);
    
    a1 = (ey.')*uy'*in*conj(ux)*ex;
    
    a3 = (eey*(eex.')); % < singular values: Ny0*Nx0 components
    a3 = (a3.^2+reg);
    
    out = vy*reshape(1./a3(:).*a1(:),Ny0,Nx0)*(vx.');
    t = t+toc;% stop the stopwatch timer - obtain the resulting time of calculation
elseif op == 2
        if (nargin<11 || isempty(reg)),  reg=[ry,rx]; end
        if reg(1) > Ny0
            disp('Error: Wrong setting of the number of signular values. reg(1) must be not larger than Ny0');
            reg(1) = ry;  disp(['Use rank(Ay)=' num2str(ry) ' singular values']);
        end
        if reg(2) > Nx0
            disp('Error: Wrong setting of the number of signular values. reg(2) must be not larger than Nx0');
            reg(2) = rx;  disp(['Use rank(Ax)=' num2str(rx) ' singular values']);
        end
    if nargout > 1,    tic;end                                                  % start a stopwatch timer 
    % SVD
    [uy,ey,vy]=svd(Aoy);eey = diag(ey); 
    [ux,ex,vx]=svd(Aox);eex = diag(ex);
    
    pinvey = diag([1./(eey(1:reg(1))).^2;zeros(Ny0-reg(1),1)]);
    pinvex = diag([1./(eex(1:reg(2))).^2;zeros(Nx0-reg(2),1)]);
    
    out = vy*pinvey*(ey.')*uy'*in*conj(ux)*ex*pinvex*vx.';
    t = t+toc;% stop the stopwatch timer - obtain the resulting time of calculation
else 
    disp('Error: Wrong setting of the operation, please choose op={0,1,2}.'); out=[]; return
end

% 2008 - 2013 (c) TUT v. 2.0